@@ -1,6 +1,6 @@
 package Reddit::Client;
 
-our $VERSION = '1.082'; 
+our $VERSION = '1.083'; 
 $VERSION = eval $VERSION;
 
 use strict;
@@ -91,6 +91,7 @@ use constant API_MY_SUBREDDITS       => 21;
 use constant API_USER                => 22;
 use constant API_SELECTFLAIR         => 23;
 use constant API_FLAIROPTS           => 24;
+use constant API_EDITWIKI            => 25;
 
 #===============================================================================
 # Parameters
@@ -105,7 +106,7 @@ our $UA               = sprintf 'Reddit::Client/%f', $VERSION;
 
 our @API;
 $API[API_ME            ] = ['GET',  '/api/v1/me'            ];
-$API[API_INFO          ] = ['GET', '/api/info'            ];
+$API[API_INFO          ] = ['GET',  '/api/info'             ];
 $API[API_SEARCH        ] = ['GET',  '/subreddits/search'    ];
 $API[API_LOGIN         ] = ['POST', '/api/login/%s'         ];
 $API[API_SUBMIT        ] = ['POST', '/api/submit'           ];
@@ -129,6 +130,7 @@ $API[API_MARKALL       ] = ['POST', '/api/read_all_messages'];
 $API[API_USER          ] = ['GET',  '/user/%s/%s'           ];
 $API[API_SELECTFLAIR   ] = ['POST', '/r/%s/api/selectflair' ];
 $API[API_FLAIROPTS     ] = ['POST', '/r/%s/api/flairselector'];
+$API[API_EDITWIKI      ] = ['POST', '/r/%s/api/wiki/edit'   ];
 #===============================================================================
 # Package routines
 #===============================================================================
@@ -306,9 +308,7 @@ sub api_json_request {
 
     my $info   = $API[$api] || croak "Unknown API: $api";
     my ($method, $path) = @$info;
-	#print "$path @$args\n";
     $path = sprintf $path, @$args;
-	#print "$path @$args\n";
 
     my ($query, $post_data);
     if ($method eq 'GET') {
@@ -318,8 +318,6 @@ sub api_json_request {
     }
 
     my $result = $self->json_request($method, $path, $query, $post_data);
-	#use Data::Dump::Color;
-	#dd $result;
 
     if (exists $result->{errors}) {
         my @errors = @{$result->{errors}};
@@ -330,8 +328,6 @@ sub api_json_request {
             croak $message;
         }
     }
-	#use Data::Dump::Color;
-	#dd $result;
 
     if (defined $callback && ref $callback eq 'CODE') {
         return $callback->($result);
@@ -464,7 +460,7 @@ sub mark_inbox_read {
 }
 
 #===============================================================================
-# Finding subreddits and listings
+# Subreddits and listings
 #===============================================================================
 
 sub info {
@@ -475,7 +471,6 @@ sub info {
     
     my $info = $self->api_json_request(
 	api => API_INFO, 
-	#args => [$id], 
 	data=>$query);
     my $rtn = $info->{data}->{children}[0]->{data};
     $rtn->{kind} = $info->{data}->{children}[0]->{kind} if $rtn;
@@ -778,6 +773,35 @@ sub get_flair_options {
 	return $result;
 }
 
+#==============================================================================
+# Subreddit management
+#==============================================================================
+
+sub edit_wiki {
+	my ($self, %param) = @_;
+	my $page 	= $param{page} || croak "Need 'page'";
+	my $content	= defined $param{content} ? $param{content} : croak "Need 'content'";
+	# Reddit maximum length is 524,288
+	if (length $content > 524288) { croak "Maximum length for 'content' is 524288 bytes."; }
+	my $sub		= $param{subreddit} || croak "Need 'subreddit'";
+	my $previous	= $param{previous};
+	my $reason	= $param{reason};
+
+	my $data	= {};
+	$data->{page}	= $page;
+	$data->{content}= $content;	
+	if ($previous) 	{ $data->{previous} = $previous; }
+	if ($reason) 	{ $data->{reason} = substr $reason, 0, 256; }
+
+	my $result = $self->api_json_request(
+		api 	=> API_EDITWIKI,
+		args 	=> [$sub],
+		data	=> $data,
+	);
+
+	return $result;
+}
+
 #===============================================================================
 # Comments
 #===============================================================================
@@ -905,11 +929,12 @@ __END__
 
 =pod
 
+
 =head1 NAME
 
 Reddit::Client - A Perl wrapper for the Reddit API.
 
-The official documentation can be found at L<http://redditclient.readthedocs.org/en/latest/>.
+B<The official documentation can be found at> L<http://redditclient.readthedocs.org/en/latest/>. What follows below might not be wrong but is not guaranteed to be right, and the likelihood of it being wrong increases with time.
 
 =head1 SYNOPSIS
 
@@ -982,7 +1007,7 @@ The official documentation can be found at L<http://redditclient.readthedocs.org
 =head1 DESCRIPTION
 
 Reddit::Client provides methods and object wrappers for objects exposed
-by the Reddit API. This module handles HTTP communication, oauth session management, and communication with Reddit's external API. For more information about the Reddit API, see L<https://github.com/reddit/reddit/wiki/API>.
+. This module handles HTTP communication, oauth session management, and communication with Reddit's external API. For more information about the Reddit API, see L<https://github.com/reddit/reddit/wiki/API>.
 
 Beginning August 3rd, 2015, the Reddit API requires Oauth authentication. This amounts to two extra arguments at the beginning of your script; in exchange you you twice as many requests per minute as before (60 vs 30) and some added convenience on the back end.
 
@@ -990,229 +1015,258 @@ To get Oauth keys, visit your apps page: L<https://www.reddit.com/prefs/apps>. C
 
 Every account that uses this script must have permission to use it. So for example if you were to check the inboxes of your various accounts, each one would need permission. To add accounts click "add developer" on the right, after you've created the app.
 
-=head1 CONSTANTS
+=head1 Methods
 
-    VIEW_HOT            "Hot" links feed
-    VIEW_NEW            "New" links feed
-    VIEW_RISING		"Rising" links feed
-    VIEW_CONTROVERSIAL  "Controversial" links feed
-    VIEW_TOP            "Top" links feed
+These are the methods of the Reddit::Client class.
 
-    VIEW_DEFAULT        Default feed if not specified (VIEW_HOT)
-    DEFAULT_LIMIT       The default number of links to be retried (25)
+Two notes about methods that return lists of things:
 
-    VOTE_UP             Up vote
-    VOTE_DOWN           Down vote
-    VOTE_NONE           Remove any vote
+1. All methods that return lists of things return them as a Perl _list_-- that is, a reference to an anonymous array. In practice this means that you get at the values with `@$array` instead of `@array`.
 
-    SUBREDDITS_HOME     List reddits on the homepage
-    SUBREDDITS_POPULAR  List popular reddits
-    SUBREDDITS_NEW      List new reddits
-    SUBREDDITS_MINE     List reddits for which the logged in user is subscribed
-    SUBREDDITS_CONTRIB  List reddits for which the logged in user is a contributor
-    SUBREDDITS_MOD      List reddits for which the logged in user is a moderator
+2. All methods that return lists of things accept three optional parameters: `limit`, `before`, and `after`. You may recognize them from your address bar when viewing pages after the front page. `limit` defaults to 25, maximum 100. If `limit`  is present but false, this is interpreted as "no limit" and the max is returned.
+
+
+ B<limit> I<integer> How many things to return. Default 25, maximum 100. If `limit`  is present but false, this is interpreted as "no limit" and the max is returned.
+
+ B<before> I<fullname> Return results that occur before `fullname`. 
+
+ B<after> I<fullname> Return results that occur after `fullname`.
 
-=head1 GLOBALS
 
 =over
 
+=item delete 
 
-=item $DEBUG
+    delete ( $fullname )
 
-When set to true, outputs a small amount of debugging information.
+Delete a post or comment.
 
+=item edit_wiki
 
-=back
+    edit_wiki ( subreddit => $subreddit, page => $page, content => $content,
+				[ previous => $previous_version_number,] [ reason => $edit_reason, ] )
+	
+	* `page` is the page being edited.
+	* `content` is the new page content. Can be empty but must be defined. Maximum 524,288 characters.
+	* `reason` is the edit reason. Max 256 characters, will be truncated if longer. Optional.
+	* `previous` is the ID of the intended previous version of the page; if provided, that is the version the page will be rolled back to in a rollback. However, there's no way to find out what this should be from the Reddit website, or currently from Reddit::Client either. Use it only if you know what you're doing. Optional.
 
-=head1 SUBROUTINES/METHODS
+=item find_subreddits
 
-=over
+    find_subreddits ( q => $query, [ sort => 'relevance' ,] 
+					[ limit => DEFAULT_LIMIT ,] [ before => undef ,] [ after => undef ,] )
 
-=item new(user_agent => ... [, client_id =>, secret =>, username=>, password =>])
+Returns a list of Subreddit objects matching the search string `$query`. Optionally sort them by `sort`, which can be "relevance" or "activity".
 
-Creates a new Reddit::Client object. Optionally authenticates at the same time. If any of the four optional arguments are used, all are required.
+=item get_comment 
 
+    get_comment ( $fullname )
 
-=item get_token(client_id =>, secret =>, username=>, password =>)
+Returns a [Comment](classes/comment) object for `$fullname`.
 
-Get an Oauth token from Reddit. The client ID and secret come  from your apps page, where you will first have to register the script: L<https://www.reddit.com/prefs/apps>. Only the account that created the app and accounts that are registered as developers for the app will be able to use it. 
+=item get_flair_options
 
-For a Perl script, select a "script" type app. The "about url" and "redirect uri" can be any address; they are not used for scripts. 
+    get_flair_options( subreddit => $subreddit, post_id => $post_id_or_fullname, username => $username )
+	
+Return the flair options for either the post or the user provided.
+
+Returns a hash containing two keys:
+
+* `choices` is an array of hash references containing the flair options. Most important is `flair_template_id`, which is used to set the flair of a post or user with set_post_flair or set_user_flair. `flair_text` contains the text of the flair.
+* `current` is a hash of the post or user's existing flair.
+
+=item get_inbox 
+
+    get_inbox ( [ view => MESSAGES_INBOX ,] [ limit => DEFAULT_LIMIT ,]
+	            [ before => undef ,] [ after => undef ,] )
+				
+Returns a list of [Message](classes/message) objects, where `view` is one of the MESSAGE [constants](constants). All arguments are optional. If all are omitted your default inbox will be returned-- what you would see if you went to reddit.com and clicked the mailbox icon.
+
+Checking your inbox via the API doesn't mark it as read. To do that you'll need to call [mark_inbox_read](http://redditclient.readthedocs.org/en/latest/main-methods/#mark_inbox_read).
+
+=item get_link 
+
+    get_link ( $fullname )
+
+Returns a [Link](classes/link) object for `$fullname`.
+
+=item get_links
+
+    get_links ( [ subreddit => undef ,] [ view => VIEW_DEFAULT ,] [ limit => DEFAULT_LIMIT ,]
+	            [ before => undef ,] [ after => undef ,] )
+
+Returns a list of [Link](classes/link) objects. All arguments are optional.
+
+`subreddit` can be a subreddit or multi (ex: "pics+funny"). If omitted, results from the user's front page will be returned-- i.e. what you would see if you visited reddit.com as that user. 
+
+`view` is a feed type [constant](constants)-- i.e. VIEW_HOT, VIEW_NEW, etc.
+		
+fetch_links is an alias for get_links.
+
+=item get_permalink
+
+    get_permalink ( $comment_id, $post_id )
+	
+Returns a permalink for `$comment_id`. **If you already have a [Comment](classes/comment) object, use its get_permalink function instead**, aka `$comment->get_permalink()`. 
+
+This version causes an extra request because it has to ask Reddit for the parent post's URL first, while a Comment object already has that information. It's provided for backwards compatibility, and for the rare case when you may have a comment's ID but not a comment object (perhaps you have a list of IDs stored in a database). It may be deprecated in the future.
+
+`$comment_id` and `$post_id` can be either fullnames or short IDs.
+
+=item get_subreddit_comments
+
+    get_subreddit_comments ( [ subreddit => '' ,] [ view => VIEW_DEFAULT ,] [ limit => DEFAULT_LIMIT ,]
+                             [ before => undef ,] [ after => undef ,]  )
+
+Returns a list of [Comment](classes/comment) objects from a subreddit or multi.
+
+All arguments are optional. If subreddit is omitted the account's "front page" subreddits are returned (i.e. what you see when you visit reddit.com and are logged in). 
+
+`view` is a feed type [constant](constants)-- i.e. VIEW_HOT, VIEW_NEW, etc.
+
+=item get_token
+
+    get_token ( client_id => $client_id, secret => $secret, username => $username, password => $password )
+
+Get an Oauth token from Reddit. This is analogous to the old login function, and can be considered identical, with the exception that, if your script runs continuously for more than an hour, a new token will be obtained hourly.
+
+=item get_user
+
+    get_user ( [ user => $username ,] [ view => USER_OVERVIEW ,]
+               [ limit => DEFAULT_LIMIT ,][ before => undef ,] [ after => undef ,]  )
+			   
+Get information about a user, where `view` is one of the [user constants](constants): overview, comments, submitted, gilded, upvoted, downvoted, hidden, or saved. Defaults to overview, which shows the user's most recent comments and posts.
+
+Note that if you try to get the upvoted, downvoted, hidden, or saved activity for a user other than yourself, Reddit will return a 403 error, so be sure to [wrap requests in a try/catch](http://redditclient.readthedocs.org/en/latest/examples/#catch-exceptions) if there's any chance that might happen. (Of course, all requests should be wrapped in a try/catch anyway, the internet being an unpredictable place.)
+
+The result can be a list of [Links](classes/link), [Comments](classes/comment), or a mix of both, depending on what `view` you requested. You can determine which is which by looking at the type property, which is "t1" for comments and "t3" for posts.
 
- 
 =item has_token
 
+    has_token()
+
 Return true if a valid Oauth token exists.
 
+=item hide
+
+    hide ( $fullname )
+
+Hide a post.
+
+=item info 
+
+    info ( $fullname )
+
+Returns a hash of information about `$fullname`. `$fullname` can be any of the 8 types of thing.
+
+=item list_subreddits
+
+    list_subreddits ( [ view => SUBREDDITS_HOME ,] [ limit => DEFAULT_LIMIT ,]
+                      [ before => undef ,] [ after => undef ,]  )
+
+Returns a list of subreddits, where `view` is one of the SUBREDDIT [constants](constants). 
+
+An alias function is provided for each view type, which is the same as calling `list_subreddits` with the view already provided:
+
+* contrib_subreddits ( [ limit => DEFAULT_LIMIT ,] [ before => undef ,] [ after => undef ,] )
+* home_subreddits ( [ limit => DEFAULT_LIMIT ,] [ before => undef ,] [ after => undef ,] )
+* mod_subreddits ( [ limit => DEFAULT_LIMIT ,] [ before => undef ,] [ after => undef ,] )
+* my_subreddits ( [ limit => DEFAULT_LIMIT ,] [ before => undef ,] [ after => undef ,] )
+* new_subreddits ( [ limit => DEFAULT_LIMIT ,] [ before => undef ,] [ after => undef ,] )
+* popular_subreddits ( [ limit => DEFAULT_LIMIT ,] [ before => undef ,] [ after => undef ,] )
+
+=item mark_inbox_read
+
+    mark_inbox_read()
+
+Mark everything in your inbox as read. May take some time to complete.
 
 =item me
 
-Returns a Reddit::Client::Account object.
+    me()
 
-=item mark_all_read
+Return an [Account](classes/account) object that contains information about the logged in account. Aside from static account information it contains the `has_mail` property, which will be true if there is anything in your inbox.
 
-Mark your inbox read. May not happen instantly.
+=item new 
 
+    new ( user_agent => $user_agent, [ client_id => $client_id, secret => $secret, 
+          username => $username, password => $password ] )
 
-=item info($fullname)
+Instantiate a Reddit::Client object. Optionally get an Oauth token (analogous to logging in) at the same time. If any of the four optional arguments are used, all are required.
 
-Returns a hash of extended information about C<$fullname>. Contains some information not available elsewhere.
+`$user_agent` is always required, and should be something descriptive that uniquely identifies your app. The [API Rules](https://github.com/reddit/reddit/wiki/API#rules) says it should be
 
+>something unique and descriptive, including the target platform, a unique application identifier, a version string, and your username as contact information
 
-=item list_subreddits($type)
+It also includes this warning:
 
-Returns a list of Reddit::Client::SubReddit objects for C<$type>, where C<$type>
-is a C<SUBREDDITS_*> constant. A number of alias functions are available:
+>**NEVER lie about your user-agent.** This includes spoofing popular browsers and spoofing other bots. We will ban liars with extreme prejudice.
 
+=item save 
 
-=over 4
+    save ( $fullname )
 
-=item my_subreddits
+Save a post or comment. 
 
-Alias for C<list_subreddits(SUBREDDITS_MINE)>. 
+=item send_message 
 
+    send_message ( to => $username, subject => $subject, text => $message )
 
-=item home_subreddits
+Send a private message to `$username`. `$subject` is limited to 100 characters.
 
-Alias for C<list_subreddits(SUBREDDITS_HOME)>. 
+=item set_post_flair
 
+    set_post_flair ( subreddit => $subreddit, post_id => $post_id_or_fullname, flair_template_id => $flair_id )
+	
+Set the flair on a post. The 'flair_template_id' is acquired via [get_flair_options](main-methods/#get_flair_options).
 
-=item mod_subreddits
+=item set_user_flair
 
-Alias for C<list_subreddits(SUBREDDITS_MOD)>. 
+    set_user_flair ( subreddit => $subreddit, username => $username, flair_template_id => $flair_id )
+	
+Set the flair for a user. The 'flair_template_id' is acquired via [get_flair_options](/main-methods/#get_flair_options).
 
+=item submit_comment 
 
-=item contrib_subreddits
+    submit_comment ( parent_id => $fullname, text => $text)
 
-Alias for C<list_subreddits(SUBREDDITS_CONTRIB)>.
+Submit a comment under `$fullname`, which must be a post or comment. Returns fullname of the new comment.
 
+=item submit_link 
 
-=item popular_subreddits
+    submit_link ( subreddit => $subreddit, title => $title, url => $url, [ inbox_replies => 1 ] )
 
-Alias for C<list_subreddits(SUBREDDITS_POPULAR)>.
+Submit a link. Returns the fullname of the new post. If `inbox_replies` is set and is false, disable inbox replies for that post.
 
+=item submit_text 
 
-=item new_subreddits
+    submit_text ( subreddit => $subreddit, title => $title, text => $text, [ inbox_replies => 1 ] )
 
-Alias for C<list_subreddits(SUBREDDITS_NEW)>.
+Submit a text post. Returns the fullname of the new post. If `inbox_replies` is set and is false, disable inbox replies for that post.
 
-=back
+=item unhide 
 
+    unhide ( $fullname )
 
-=item find_subreddits($query)
+Unhide a post.
 
-Returns a list of SubReddit objects matching C<$query>.
+=item unsave 
 
+    unsave ( $fullname )
 
-=item fetch_links([subreddit => ...], [view => ...], [limit => ...], [before => ...], [after => ...])
+Unsave a post or comment.
 
-Returns a list of Link objects. 
+=item version
 
-All arguments are optional. If C<subreddit> is specified,
-the list of links is returned from the desired subreddit or multi (i.e. "pics+new"). Otherwise, the
-links will be from the front page of the authenticated user (i.e. what you would see as the front page if you logged in as that user) . C<view> specifieds the feed (e.g.
-C<VIEW_NEW> or C<VIEW_HOT>). C<limit> may be used to limit the number of
-objects returned; Reddit returns a maximum of 100. C<before> and C<after> denote the placeholders for
-slicing the feed up, just as the reddit urls themselves do. Data is returned
-as a hash with three keys, I<before>, I<after>, and I<items>.
+    version()
 
+Return the Reddit::Client version.
 
-=item delete(name => ...)
+=item vote 
 
-Deletes a post or comment. The object's fullname is required.
-
-
-=item submit_link(subreddit => ..., title => ..., url => ... [inbox_replies => ...])
-
-Submits a link to a reddit. Returns the fullname of the new link. If inbox_replies is set and is false, disable inbox replies for that post.
-
-
-=item submit_text(subreddit => ..., title => ..., text => ... [inbox_replies => ...)
-
-Submits a self-post to a reddit. Returns the fullname of the new post. If inbox_replies is set and is false, disable inbox replies for that post.
- 
-
-=item get_subreddit_comments([subreddit => ...][before => ...][after => ...][limit => ...])
-
-Return a list of Reddit::Client::Comment objects from a subreddit or multi. All arguments are optional. If subreddit is ommitted, a multi of the subscribed subreddits from the authenticating account will be returned (i.e. what you see when you visit reddit.com's from page and are logged in). If limit is ommitted, Reddit's default limit of 25 is used. If limit is present but false, this is interpreted as no limit and the maximum is returned (100).
-
-
-=item submit_comment(parent_id => ..., text => ...)
-
-Submits a new comment underneath C<parent_id>. C<parent_id> must be a fullname.
-
-
-=item vote(item_id => ..., direction => ...)
-
-Votes for C<item_id>. C<direction> is one of C<VOTE_UP>, C<VOTE_DOWN>,
-or C<VOTE_NONE>. C<item_id> must be a fullname.
-
-
-=item save($item_id)
-
-Saves C<$item_id> under the user's account. C<item_id> must be a fullname.
-
-
-=item unsave($item_id)
-
-Unsaves C<$item_id> under the user's account. C<item_id> must be a fullname.
-
-
-=item hide($item_id)
-
-Hides $<item_id>. Throws an error if the user does not have permission to hide
-the item in question. C<item_id> must be a fullname.
-
-
-=item unhide($item_id)
-
-Unhides $<item_id>. Throws an error if the user does not have permission to
-unhide the item in question. C<item_id> must be a fullname.
-
-
-=back
-
-=head1 INTERNAL ROUTINES
-
-=over
-
-=item DEBUG
-
-When C<$Reddit::Client::DEBUG> is true, acts as syntactic sugar for
-warn(sprintf(@_)). Used to provided logging.
-
-
-
-=item subreddit
-
-Strips slashes and leading /r from a subreddit to ensure that only
-the "display name" of the subreddit is returned.
-
-
-=item request
-
-Performs a request to reddit's servers using LWP. If the user is
-logged in, adds the "uh" and "modhash" parameters to POST queries
-as well as adding the reddit-specified cookie value for reddit_session.
-
-
-=item json_request
-
-Wraps C<request>, configuring the parameters to perform the request
-with an api_type of "json". After the request is complete, parses the
-JSON result and throws and error if one is specified in the result
-contents. Otherwise, returns the json data portion of the result.
-
-=item    get_permalink
-
-=item    get_token
-
-=item    send_message
-
-=item api_json_request
-
-Wraps C<json_request>, getting method and path from an API_CONSTANT.
+    vote ( $fullname, $direction )
 
+Vote on a post or comment. Direction must be 1, 0, or -1 (0 to clear votes).
 
 =back
 
@@ -1220,9 +1274,10 @@ Wraps C<json_request>, getting method and path from an API_CONSTANT.
 
 L<mailto:earth-tone@ubwg.net>
 
-
 =head1 LICENSE
 
 BSD license
 
 =cut
+
+
